;**********************Procedure arc_cal version 1_0********************
;Procedure  arc_cal,c,h,f,spectra,yoff,del,wlmin,wlmax
;
;Function   For each fiber fit a polynomial of wavelength vs pixel number
;           using the header table of pixel range and  wavelengths for selected arclines
;           use polymonial to fill the wavelength per pixel in the intermediate arc spectra
;
;Input:    arcfile header and intermediate arc spectra (c,h,f,spectra)
;           yoff - relative offset of individual arc spectra
;Output:    wavelength calibration added data array of arc spectra (spectra)
;           using polynomial fit of wavelength vs pixel number
;           array of offsets between calibration data and polynomial fits (del)
;           limits of common wavelegth range found from spectra (wlmin,wlmax)
;
;Error     FATAL ERROR insufficient arclines flagged for spectified order of fit
;message
;Warning   Warning Bad Wavelength solution for fiber no. ### at 
;          arcline no. ###
;
;Procedures called;
;printl       - Prints text message to log file
;
;******************Procedure********************************************
pro arc_cal,c,h,f,spectra,yoff,del,wlmin,wlmax

;******evalue the size of frame in y **********************************
siz_y=fix(c.ymax-c.yzero)

;**********check number of arclines flagged is OK***********************
index = where(c.aflag gt 0,nset)
if nset lt abs(c.arcorder) + 1 then printl,c.log,$
'FATAL ERROR insufficient arclines flagged for spectified order of fit'
if nset lt abs(c.arcorder) + 1 then stop
;***********set up arrays used in calculation***************************
ospec = dblarr(c.nfiber,siz_y/c.ybin)
;array yval holds the centerline of each calibration line for each fiber
yval  = dblarr(c.nfiber,c.narcline)
ind   = indgen(siz_y/c.ybin,/double)
delta = dblarr(c.nfiber)
del= fltarr(c.nfiber,nset)
std= fltarr(2,c.nfiber)

;*********set y axis range for plots************************************
yscale = 2*float(c.ccdzero)*(c.lwide-4)/4.0
if c.arcscale gt 1 then yscale = yscale/float(c.arcscale)
;*****************shift spectra by y offset*****************************
specoff = fltarr(c.nfiber,siz_y/c.ybin+max(yoff))
pixel   = indgen(siz_y/c.ybin+max(yoff))-max(yoff)
for i = 0,c.nfiber-1 do if f(i).line gt 0 and f(i).type ne 'X' then $
  specoff(i,max(yoff)-yoff(i):max(yoff)-yoff(i)+siz_y/c.ybin-1)=$
            spectra(8*(f(i).line-1)+5,*)
;******************plot overlay of offset spectra***********************
;****************lines indicating central range of plot*****************
mark = float(pixel*0)
mark(where(pixel lt 0 or pixel gt (siz_y/c.ybin-max(yoff)))) = yscale
;***********Plot axes for overlay of arc spectra************************
plot,pixel,mark,background=c.white,color=c.black,xstyle=1$
     ,title='Overlay of offset spectra using reference line'$
     ,ytitle='e-/pixel',yrange=[0,yscale],ystyle=1,linestyle=1
;***add line markers bracketing arc lines used for calibration*********
mark = fltarr(siz_y/c.ybin+max(yoff))
for i = 0,c.narcline-1 do if c.apixc(i)-c.apixw(i) gt 1 and $
   c.apixc(i)+c.apixw(i)+2 lt siz_y+max(yoff) and c.aflag(i) eq 1 then begin
   mark(c.apixc(i)-c.apixw(i)-1:c.apixc(i)-c.apixw(i)+1) = yscale/2
   mark(c.apixc(i)+c.apixw(i)-1:c.apixc(i)+c.apixw(i)+1) = yscale/2
endif
oplot,indgen(siz_y+max(yoff)),mark,color=c.red
;*******************plot arc spectra***********************************
for i = 0,c.nfiber-1 do  oplot,pixel,specoff(i,*),color=c.black
;********************Get position of calibration lines*****************
;by fitting a Gaussian profile to sections of the spectra bracketing each line
;*************set array of pixels used for Gaussian fit***************
xpix = indgen(2*fix(c.arcwide/2)+1)-c.arcwide/2-(c.arcwide mod 2)
;**************set axes for plot showing overlay of peak profiles*****
;plot,xpix,xpix*0,yrange=[0,1],background=c.white,xstyle=1,$
;     title = 'Overlay of calibration line profiles for file '+$
;       strtrim(c.arcfile,2),ytitle='normalized',color=c.black
;**************loop for all valid fiber numbers**************************
for i = 0,c.nfiber-1 do if f(i).line gt 0 and f(i).type ne 'X' then begin
;**********select offset spectra as 1D array****************************
    ospec(i,0:siz_y/c.ybin-max(yoff)-1)=$
      spectra(8*f(i).line-8+5,yoff(i):siz_y/c.ybin+yoff(i)-max(yoff)-1)
;**********loop for each calibration line********************************
   for j = 0,c.narcline-1 do if c.aflag(j) eq 1 then begin
;**find position of peak in arc spectra corresponding to calbration line*
      maxl =max(ospec(i,c.apixc(j)-c.apixw(j):c.apixc(j)+c.apixw(j)),max_subscript)
      ymax = max_subscript  + c.apixc(j)-c.apixw(j)
;***********plot section of arc spectra used to identify the peak********
      x = ind((ymax-c.arcwide):(ymax+c.arcwide))
      y = double(ospec(i,(ymax-c.arcwide):(ymax+c.arcwide))) 
      ;oplot,xpix,y(c.arcwide/2:3*c.arcwide/2)/max(y),color=c.red

;;***fit Gaussian to get best fit to line centre using procedure MPFITPEAK
	yfit = MPFITPEAK(x, y, A, CHISQ=CHISQ, DOF=DOF, yerror=yerror, perror=perror)
	yval(i,j)=yoff(i)+A[1]
   endif

endif
;********select flagged arclines to used for polynomial fit*************
yvalf = dblarr(c.nfiber,nset)
refwl = dblarr(nset)
refpx = dblarr(nset)
j = 0
for i = 0,c.narcline-1 do if c.aflag(i) eq 1 then begin
   refwl(j) = c.acalwl(i)
   yvalf(*,j) = yval(*,i)
   j = j + 1
endif


;;;;********************** NON interactive mode ********************************;;

if c.arcorder gt 0 then begin
	for i = 0,c.nfiber-1 do if f(i).line gt 0 and f(i).type ne 'X'  then begin
	   refpx(*) = yvalf(i,*)
	;;***************** use svdfit to find wl solution *******************;;
	;   A = SVDFIT(refpx,refwl,c.arcorder,/double,yfit=YFIT)
	;   wlest = float(poly(ind,A))
	;;**********new fitting method more resistant to outliers*************;;
	   A=ROBUST_POLY_FIT(refpx,refwl,abs(c.arcorder) ,yfit,sig, /double )       
	   wlest = float(poly(ind,A))
	;****save value of delta between polynomial fit and actual line centre*****
	   del(i,*) = (yfit-refwl)
	;****keep the standard deviation of residuals for each fiber**********;;   
	   std(*,i)= [i+1,sig]
	;save the wavelength value for each pixel in the intermediate spectra array
	   spectra(8*(f(i).line-1),*) = float(poly(ind,A))
	endif
endif



;;;;;;;;;;;;;;;;;;;;;;;;;;;; Interactive fitting mode;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
if c.arcorder lt 0 then begin
window , xsize=fix(c.psize/1.5), ysize=fix(c.psize/1.9)
!P.MULTI = [0,1,1,0,0]
keyp='n'
	for i = 0,c.nfiber-1 do if f(i).line gt 0 and f(i).type ne 'X'  then begin
	   refpx(*) = yvalf(i,*)
	;;**********new fitting method more resistant to outliers*************;;
	   A=ROBUST_POLY_FIT(refpx,refwl,abs(c.arcorder) ,yfit,sig, /double )       
	   wlest = float(poly(ind,A))
	;****save value of delta between polynomial fit and actual line centre*****
	   del(i,*) = (yfit-refwl)
	;****keep the standard deviation of residuals for each fiber**********;;   
       std(*,i)= [i+1,sig]
	;save the wavelength value for each pixel in the intermediate spectra array
	   spectra(8*(f(i).line-1),*) = float(poly(ind,A))

	if (keyp eq 'N') or (keyp eq 'n') then begin
		plot, refpx,refwl,psym=4, $
		title='Fiber No. '+strtrim(i+1,2)+' Interactive Wavelength Calibration Check',$
		color=c.black, xrange=[0.0,(siz_y/c.ybin)],$
		yrange=[c.wlmin,c.wlmax], xstyle=1, ystyle=1,$
		background=c.white, xtitle='Pixel', ytitle='Wavelength',charsize=1.0
		
		oplot,ind , float(poly(ind,A)),color=c.red
		print, '----------------------------------------------------'
		print, ''
		print, 'FIber number: '+strtrim(i+1,2)
		print, 'The "STDDEV" of fitting residual: '+strtrim(sig,2)
		read, keyp, PROMPT='Enter "n" to see the next or "q" to exit calibration '+$
		'(Press any other key to continue in non-interactive mode): '
	endif
	if (keyp eq 'Q') or (keyp eq 'q') then stop
	endif
print, 'Leaving the interactive mode!'
endif	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;reset the window;;;;;;;;;;;;;;;;
window , xsize=fix(c.psize/1.2), ysize=fix(c.psize/1.4)
!P.MULTI = [0,1,3,0,0]

;*************set array of pixels used for Gaussian fit***************
xpixp = indgen(2*fix(c.arcwide/2)+1)-c.arcwide/2-(c.arcwide mod 2)
;**************set axes for plot showing overlay of peak profiles*****
plot,xpixp,xpixp*0,yrange=[0,1],background=c.white,xstyle=1,$
title = 'Overlay of calibration line profiles for file '+$
strtrim(c.arcfile,2),ytitle='normalized',color=c.black
for k = 0,c.nfiber-1 do if f(k).line gt 0 and f(k).type ne 'X' then begin  
   for j = 0,c.narcline-1 do if c.aflag(j) eq 1 then begin
;**find position of peak in arc spectra corresponding to calbration line*
      maxlp =max(ospec(k,c.apixc(j)-c.apixw(j):c.apixc(j)+c.apixw(j)),max_subscriptp)
      ymaxp = max_subscriptp  + c.apixc(j)-c.apixw(j)
;***********plot section of arc spectra used to identify the peak********
      xp = ind((ymaxp-c.arcwide):(ymaxp+c.arcwide))
      yp = double(ospec(k,(ymaxp-c.arcwide):(ymaxp+c.arcwide))) 
      yfitp = MPFITPEAK(xp, yp, AP, CHISQ=CHISQP, DOF=DOF, yerror=yerror, perror=perror)
      oplot,xpixp,yfitp(c.arcwide/2:3*c.arcwide/2)/max(yfitp),color=c.red
   endif
endif


;*************plot the standard deviation of residuals for fiber wave. calib****;;
plot, std(0,*),std(1,*) ,xstyle=1, psym=4, background=c.white, color=c.black,$
title='Standard deviation of Wavelength solution fitting residual',xtitle='Fiber Number',$
ytitle='STDDEV of residuals',xtickinterval=5

if n_elements(std(1,*)) gt 3.0 then begin
std_std=stddev(std(1,*))
std_bad_finder=where(std(1,*) gt (4.0*std_std) or std(1,*) gt (2.0/c.ybin), std_bad_cnt)
if std_bad_cnt gt 0 then begin 
bad_wc=std(0,std_bad_finder)
print, ''
printl,c.log, 'Warning: Bad Wavelength solution for Fibers: '+strtrim(fix(bad_wc),2)+$
'   TYPE: '+strtrim (f(std(0,std_bad_finder)-1).type,2)
print, ''
endif
endif

;*********Get maximum common wavelength range******************************
wllo = strarr(c.nfiber)
wlhi = strarr(c.nfiber)
for i = 0,c.nfiber-1 do if f(i).line gt 0 and f(i).type ne 'X' then begin
   wllo(i) = spectra(f(i).line*8-8,0)
   wlhi(i) = spectra(f(i).line*8-8,siz_y/c.ybin-1)
endif
wlmin = (10*long(max(wllo(where(wllo gt 0)))))/10.0+0.1
wlmax = (10*long(min(wlhi(where(wlhi gt 0)))))/10.0-0.1
;********* set up plot showing overlay of wavelength calibrated spectra****
tit = 'Common wavelength '+string(wlmin,format='(f8.1)')$
   + ' to '+string(wlmax,format='(f8.1)')+' A'+$
   '   max delta of fit:'+string(max(abs(del)),format='(f6.3)')+' pixel'
plot, spectra(0,*),spectra(0,*)*0,background=c.white,color=c.black,xstyle=1,$
       yrange=[0,yscale],ystyle=1,xrange=[c.wlmin,c.wlmax],$
      xtitle = 'Wavelength (A)',ytitle='e-/pixel',title = tit
;***plot markers  showing arclines used for calibration********************
for i = 0,nset-1 do oplot,[refwl(i),refwl(i)],[0,yscale],color=c.black,linestyle=1
;*********************plot calibrated spectra******************************
siz = size(spectra)
for i = 0,siz(1)/(c.lwide/c.xbin+4)-1 do if f(i).line gt 0 and $
  f(i).type ne 'X' then  oplot,spectra(8*i,*),spectra(8*i+1,*),color=c.black
;*********annotate plot wit values of rms fit for each calibration line****
index = where(f.type ne 'X' and f.line gt 0,nfiber)
rmsdel =  sqrt(total(del*del,1)/float(nfiber))
for i = 0,nset-1 do  XYOUTS,refwl(i)+0.015*(c.wlmax-c.wlmin),0.3*yscale,$
orientation=90,string(rmsdel(i),format='(f6.3)'),color=c.black,charsize=0.8
end
