;**************Procedure ext_file version 1_0*********************
;Procedure   extr_file,nfile,nit,h,f,spectra
;Function	Optimal extraction of spectra for a targets in a star file
;
;Input	     stafile number (1234567), number of repeat iterations of optimal extraction process
;            if fprob keyword set then use master flat (f****.fit) as probability function
;            if sprob keyword set then use master star (ns****.fit) as probability function
;Output	  header data (h,f) and intermediate spectra array for starfiles (s1234567.fit)
;
;Report to log	Extracting file r1234567 (Repeated for each target)
;           	FLAT    Iteration 0   masked points = ###  (etc.)
;           	TARGET   Iteration 0   masked points = ### (etc.)

;Error    ERROR: Saturation of extracted image (>99% full scale)
;message WARNING: Flat signal zero over part of spectra - a total of ## bad pixels masked
;                 WARNING: high extracted image, ## of full scale
;
;Procedures called;
;read_control - Returns control data in structure c
;file_debias  - Reads pipeline header (c,h,f) and debiased image from data file
;file_offset  - offset image to centre of each line  at a uniform x value
;printl       - Prints text message to log file
;
;***************************Procedure*********************************
pro extr_file,nfile,nit,h,f,spectra,fprob=fprob, sprob=sprob

;***************Read control data in structure c**********************
read_control,c
printl,c.log,'Extracting file r'+strtrim(nfile,2)
;******evalue the size of frame in y **********************************
siz_y=fix(c.ymax-c.yzero)

;****************Get input and associated files***********************
;read circfile header (hc,fc) to get line No. and attenuation for each each fiber
;read  image (offset) which gives offset of lines on CCD image
read_file,c.pdir+c.pint+'c'+strtrim(c.circfile,2),cc,hc,fc,offset
siz = size(offset)
;************get number of lines (traces) on CCD image****************
nline = siz(1)/(c.lwide/c.xbin+4)
;**********read the starfile header (h,f) and debiased image**********
file_debias,nfile,c,h,f,dummy
;*offset image (star) with centres of each line  at a constant x value
file_offset,c,h,dummy,offset,star
;******read the intermediate mask image (if required)*****************
if c.nmask gt 0 then begin
    read_file,c.pdir+c.pint+'m'+strtrim(c.maskfile,2),cm,hm,fm,dummy
;*offset image (mask) with centres of each line  at a constant x value
    file_offset,c,h,dummy,offset,mask
endif else mask = fix(star*0+1)
;**********read the median  flat file image***************************
read_file,c.pdir+c.pint+'f'+strtrim(c.flatfile,2),cf,hf,ff,dummy
;*offset image (flat) with centres of each line  at a constant x value
file_offset,c,hf,dummy,offset,flat
;**************check star signal level******************************
;*****measure peak level in (ystep) cuts through the flat image*****
;*****cuts averaged over several pixels in wavelength direction*****
ystep = fix(siz_y/c.ybin/fix(c.circlev+1))
pklev = fltarr(c.circlev)
for i = 0,c.circlev - 1 do pklev(i) = max(median(dimension=2,$
         star(*,ystep*(i+1)-c.yavg/c.ybin:ystep*(i+1)+c.yavg/c.ybin)))
;******express peak value as a fraction of full scale***************
pklev = pklev/(2.0*float(c.ccdzero))
;**Error if normalised peak is saturated or  specified value*********
if max(pklev) ge 0.99 then printl,c.log,$
  'ERROR: saturation of extracted image (>99% full scale)'
if max(pklev) ge c.sighi then printl,c.log,'WARNING: high extracted image '$
+string(max(pklev),format='(f5.2)')+' of full scale'
;Copy circfile values of line No. and attenuation to starfile header
f.line = fc.line
f.gain = fc.gain
;*Scale starfile and flat by CCD gain to give output in e- per pixel
star = star/c.ccdgain
flat = flat/c.ccdgain
;***********set up variance arrays for star and flat****************
vars = abs(star) + c.ccdron*c.ccdron
varf = abs(flat) + c.ccdron*c.ccdron
index = where(mask lt 0.1,count)
if count gt 0 then begin
   vars(index) = c.ccdron*c.ccdron
   varf(index) = c.ccdron*c.ccdron
   
endif
;***********Set up extraction profile*******************************
;get width of strip encompassing a line (trace) on the CCD
wide = c.lwide/c.xbin+4
;*********get profile in case of a top hat profile (non standard)***
if c.profile eq 0 then begin
   prob = flat*0.0
   for i=0,nline-1 do prob(i*wide+c.lwide/c.xbin/2+2-c.boxwide/c.xbin/2$
                  :i*wide+c.lwide/c.xbin/2+2+c.boxwide/c.xbin/2,*)=1.0
   endif
;*********get profile for optimal extraction************************
siz = size(dummy)
if c.profile gt 0 then begin

    ;************ using master flat to create probf profile *****************
	   ;*********smooth flat profile in the wavelength direction************
	   for j=0,siz(1)-1 do dummy(j,*) = smooth(dummy(j,*),c.yavg/c.ybin)
	   ;*offset profile (prob) with centres of each line  at a constant x value
	   file_offset,c,hf,dummy,offset,probf

    ;************ using master star to create probs profile *****************	
	read_file,c.pdir+c.pint+'ns'+strtrim(c.starfile,2),cns,hns,fns,dummy_ns
	siz_ns = size(dummy_ns)
    ;*********smooth star profile in the wavelength direction****************
    for j=0,siz_ns(1)-1 do dummy_ns(j,*) = smooth(dummy_ns(j,*),c.yavg/c.ybin)
    ;*offset image (flat) with centres of each line  at a constant x value
    file_offset,c,h,dummy_ns,offset,probs


endif

;;;**************** if c.profile=0 *********************;
if (c.profile eq 0) then begin
	 probs=prob
	 probf=prob
	 printl,c.log, 'Extraction using Top-hat (non-standard) profile'	
endif
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



;*******set target type as a function of line (trace) number********
type = strarr(nline)
for i = 0,c.nfiber-1 do if fc(i).line gt 0 then $
   type(fc(i).line-1)=f(i).type
;************Subtract scattered light (iff required)****************
;The offset image contains the median signal level between traces***
;These levels are liniarly interpolated over the width of the trace*
;to estimate the background level of scattered light for an image***
if c.nscatter eq 1 then begin
;define array of scattered light for star, flat and probability image
   scats = fltarr(wide*nline,siz_y/c.ybin)
   scatf = fltarr(wide*nline,siz_y/c.ybin)
   scatps = fltarr(wide*nline,siz_y/c.ybin)
   scatpf = fltarr(wide*nline,siz_y/c.ybin)

   for i = 0,nline-1 do begin
      for j = 0,wide-4 do begin
         scats(i*wide+j+2,*)=star((i*wide+1),*)+$
           (star(((i+1)*wide-1),*)-star((i*wide+1),*))*float(j)/(wide-4)
         scatf(i*wide+j+2,*)=flat((i*wide+1),*)+$
           (flat(((i+1)*wide-1),*)-flat((i*wide+1),*))*float(j)/(wide-4)
         scatps(i*wide+j+2,*)=probs((i*wide+1),*)+$
           (probs(((i+1)*wide-1),*)-probs((i*wide+1),*))*float(j)/(wide-4)
         scatpf(i*wide+j+2,*)=probf((i*wide+1),*)+$
           (probf(((i+1)*wide-1),*)-probf((i*wide+1),*))*float(j)/(wide-4)
      endfor
   endfor
;subtract scattered light array from star, flat and probability image
   star = star - scats
   flat = flat - scatf
   probs = probs - scatps
   probf = probf - scatpf
endif
;**********set miminun value of these arrays to zero******************
index = where(star lt 0,count)
if count gt 0 then star(index) = 0.0
index = where(flat lt 0,count)
if count gt 0 then flat(index) = 0.0
index = where(probs lt 0,count)
if count gt 0 then probs(index) = 0.0
index = where(probf lt 0,count)
if count gt 0 then probf(index) = 0.0
;************************normalise probability profile*****************
ptotals = fltarr(siz_y/c.ybin)
ptotalf = fltarr(siz_y/c.ybin)
;***set up probabilty mask where pmask=1 if probability profile is valid*
;*to extract the spectra at a given y-pixel position of a given spectra
pmasks  = intarr(nline,siz_y/c.ybin)+1
pmaskf  = intarr(nline,siz_y/c.ybin)+1

;**Check and normalise probability profile at all valid line positions**
for i = 0,nline-1 do if not strcmp(type(i),'X') then begin
;******sum probability profile across width of trace*******************
  
  ;;;;;;;;for star prob;;;;;;;;;;;;;
   ptotals(*) = total(probs(i*wide+2:(i+1)*wide-2,*),1)
   index  = where(ptotals eq 0,nzeros)
;**for y-pixel positions where sum is zero set probability mask to zero**
   if nzeros gt 0 then ptotals(index) = 1.0
   if nzeros gt 0 then pmasks(i,index) = 0
   for j = 2,wide-2 do probs(i*wide+j,*) = probs(i*wide+j,*)/ptotals(*)
   
   ;;;;;;;;for flat prob;;;;;;;;;;;;;
    ptotalf(*) = total(probf(i*wide+2:(i+1)*wide-2,*),1)
   index  = where(ptotalf eq 0,nzerof)
;**for y-pixel positions where sum is zero set probability mask to zero**
   if nzerof gt 0 then ptotalf(index) = 1.0
   if nzerof gt 0 then pmaskf(i,index) = 0
   for j = 2,wide-2 do probf(i*wide+j,*) = probf(i*wide+j,*)/ptotalf(*)
     
endif


;Print log message if any pixels are masked for bad probabilty profile
index  = where(pmasks eq 0,nzero)
if nzero then printl,c.log,'WARNING: Flat signal zero over part of spectra -'+$
         ' a total of '+strtrim(nzero,2)+' bad pixels masked'
index  = where(pmaskf eq 0,nzero)
if nzero then printl,c.log,'WARNING: Star signal zero over part of spectra -'+$
         ' a total of '+strtrim(nzero,2)+' bad pixels masked'         
         
         
;**********************Extract spectra ********************************
;***************Calling extr_make**************************************
;Input:  Flat file header data (c,hf) flat image (flat) and variance (varf)
;        mask (mask),profile (prob) , alternative profile (probf),
;        target types (type) and No. iterations (nit)
;Output: line (extracted) spectra of flat (lspecf) and variance (vspecf)
;***************Calling extr_make**************************************
;Input:  Flat file header data (c,h) flat image (star) and variance (vars)
;        mask (mask),profile (prob), alternative profile (probf or probs)
;        target types (type) and No. iterations (nit)
;Output: line (extracted) spectra of star (lspecs) and variance (vspecs)

if (c.profile eq 0) then begin
;;; if profile=0 then probs and probf are set to prob, earlier;;;;
extr_make,c,hf,flat,varf,mask,probf,probf,type,nit,lspecf,vspecf
extr_make,c,h,star,vars,mask,probs,probs,type,nit,lspecs,vspecs
endif

if (c.profile eq 1) or (keyword_set(fprob)) then begin
printl,c.log, 'Optimal extraction using MASTER_FLAT profile'
extr_make,c,hf,flat,varf,mask,probf,probf,type,nit,lspecf,vspecf
extr_make,c,h,star,vars,mask,probf,probf,type,nit,lspecs,vspecs
endif

if (c.profile eq 2) and keyword_set(sprob) then begin
printl,c.log, 'Optimal extraction using MASTER_STAR profile'
extr_make,c,hf,flat,varf,mask,probf,probf,type,nit,lspecf,vspecf
extr_make,c,h,star,vars,mask,probs,probs,type,nit,lspecs,vspecs
endif

if (c.profile eq 3) and keyword_set(sprob) then begin
printl,c.log, 'Optimal extraction using MASTER_STAR+(alternative:Flat) profile'
extr_make,c,hf,flat,varf,mask,probf,probf,type,nit,lspecf,vspecf
extr_make,c,h,star,vars,mask,probs,probf,type,nit,lspecs,vspecs
endif

;Load extracted spectra of flat and star into intermediate spectra array
;each line (trace) is represented by 8 columns (0 to 7 etc) of the array
;***column 0 is zero (later it will hold  wavelength vs pixel No.)*****
spectra = fltarr(8*nline,siz_y/c.ybin)


;!P.MULTI=[0,1,3]
;xxx = ''
for i = 0,nline-1 do if not strcmp(type(i),'X') then begin

;**************calculate pixel by pixel variation of flat *******;
;**************by dividing the flat to the DC component**********;

;;;;;; programmer note: we have to edit min_sn_flat later;;;;;;;;

;;;******* in order to avoid zero/zero , first filter data******;
fl=lspecf(i,*)
smt_fl=smooth(fl,(c.flatsmth/c.ybin), /EDGE_TRUNCATE)
low_fl=where(fl lt 1.0,clf)
if clf gt 0 then fl(low_fl)=1.0
low_smt_fl=where(smt_fl lt 1.0,clsf)
if clsf gt 0 then smt_fl(low_smt_fl)=1.0

;;;***************create norm flat********************************;
norm_flat=fl/smt_fl


;;********use boxcar to find s/n of profile and filter it*********;
ja_f=0 & step=floor(c.flatsmth/c.ybin) & snflat=80.0
while (ja_f+step lt n_elements(norm_flat)-1.0) do begin
if abs(mean(norm_flat(ja_f:ja_f+step)))/stddev(norm_flat(ja_f:ja_f+step)) lt snflat then $
norm_flat(ja_f:ja_f+step)=1.0
ja_f=ja_f+step
endwhile

;;*******replace first and last section of profile with one*******;
norm_flat((n_elements(norm_flat)-1.0-step):(n_elements(norm_flat)-1.0))=1.0

norm_flat=(norm_flat*0.0)+1.0



;cgplot,fl , ystyle=1, color='red', thick=1
;cgplot,smt_fl , ystyle=1, color='blue', thick=1, /overplot
;cgplot,norm_flat , ystyle=1, color='blue', thick=1
;;cgplot,smt_fl , ystyle=1, color='blue', thick=1
;cgplot, lspecs(i,*)/float(norm_flat), ystyle=1, color='black'
;READ, xxx, PROMPT='Enter Name:'



;***columns 1 and 2 hold the magnitude an variance of the star spectra
   spectra(i*8+1,*) = lspecs(i,*)
   spectra(i*8+2,*) = vspecs(i,*)
;***columns 3 and 4 hold the magnitude an variance of the flat spectra
   spectra(i*8+3,*) = lspecf(i,*)
   spectra(i*8+4,*) = vspecf(i,*)
;*************get mean magnitude of (unmasked) flat spectra**************
   index  = where(lspecf(i,*) gt 0.1,count)
   if count gt 0 then meanf = total(lspecf(i,index))/float(count) $
      else meanf=mean(lspecf)
;*********valid pixel positions of star spectrum*************************
;**********these are unmasked points with a good extraction profile******
;   index = where((lspecf(i,*) gt 0.01) and (lspecs(i,*) gt 0.01) $
;                 and (pmasks(i,*) gt 0) and (pmaskf(i,*) gt 0) ,count)
;;***************** Old Flat-Fielding method *****************************

;*columns 5 and 6 hold the magnitude an variance of the normalise spectra
;*****which is (star spectrum/flat spectrum) x mean flat signal***********
;   if count gt 0 then begin
;      spectra(i*8+5,index) = lspecs(i,index)*meanf/lspecf(i,index)
;      spectra(i*8+6,index) = vspecs(i,index)*(meanf/lspecf(i,index))*$
;                                             (meanf/lspecf(i,index))
;      spectra(i*8+7,index) = pmask(i,index)
;   endif else begin
;      spectra(i*8+5,*) = lspecs(i,*)* mflat/lspecf(i,*)
;      spectra(i*8+6,*) = vspecs(i,*)*mflat*mflat/lspecf(i,*)/lspecf(i,*)*$
;                                            (mflat/lspecf(i,*))
;      spectra(i*8+7,*) = pmask(i,*)
;   endelse

;; ****************** new Flat-Fielding method p. by p. variation**;
;*columns 5 and 6 hold the magnitude an variance of the normalise spectra
spectra(i*8+5,*) =lspecs(i,*)/float(norm_flat)
spectra(i*8+6,*) =vspecs(i,*)/(float(norm_flat)/float(norm_flat))
;*columns 7 holds the related mask
spectra(i*8+7,*) = pmaskf(i,*)

endif
end